/*
 * Copyright
 *   2006 axYus - www.axyus.com
 *   2006 JP.Tessier- jean-philippe.tessier@axyus.com
 *
 * This file is part of XEMELIOS.
 *
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fr.gouv.finances.cp.xemelios.importers;

import fr.gouv.finances.cp.utils.Pair;
import fr.gouv.finances.cp.xemelios.common.ToolException;
import fr.gouv.finances.cp.xemelios.common.config.DocumentModel;

/**
 * Definit le contrat de service pour les importeurs.
 * Ce sont les differentes implementations de cette interface
 * qui interagiront avec l'utilisateur via une interface swing,
 * une ligne de commande, un webservice, etc...
 */

public interface ImportServiceProvider {
	public final int MSG_TYPE_INFORMATION = 1;
	public final int MSG_TYPE_ERROR = 2;
	
	public Pair getBudget(DocumentModel dm, Pair collectivite, String fileName) throws ToolException;
	public Pair getCollectivite(DocumentModel dm, String fileName) throws ToolException;
	public boolean getOverwrite(String docName) throws Exception;
	public void pushCurrentProgress(int progress) throws NullPointerException;
	public void displayProgress(final int nbFiles);
	public void displayMessage(String msg, int msgType);
	public void displayException(Throwable t);
//	public void budgetInteractif(boolean value);
//	public void collectiviteInteractif(boolean value);
	public void startLongWait();
	public void endLongWait();
    public void setDisplayFeedback(boolean display);
    public boolean shouldDisplayFeedback();
    public void setCollectivite(Pair p);
    public void setBudget(Pair p);
}
