/*
 * Copyright 
 *   2007 axYus - www.axyus.com
 *   2007 C.Marchand - christophe.marchand@axyus.com
 * 
 * This file is part of XEMELIOS.
 * 
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fr.gouv.finances.cp.xemelios.updater.config;

import java.util.ArrayList;
import java.util.Iterator;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import fr.gouv.finances.cp.utils.xml.marshal.InvalidXmlDefinition;
import fr.gouv.finances.cp.utils.xml.marshal.XmlAttributes;
import fr.gouv.finances.cp.utils.xml.marshal.XmlMarshallable;
import fr.gouv.finances.cp.utils.xml.marshal.XmlOutputter;
import javax.xml.namespace.QName;

public class ConfigurationModel implements XmlMarshallable {
	public static final transient String TAG="configuration";
        public static final transient QName QN = new QName(TAG);
	
	private String className;
	private ArrayList<ParamModel> params;

	public ConfigurationModel(QName tagName) {
		super();
		params = new ArrayList<ParamModel>();
	}

	public void addCharacterData(String cData) throws SAXException {
	}

	public void addChild(XmlMarshallable child, QName tagName) throws SAXException {
		if(ParamModel.QN.equals(tagName)) params.add((ParamModel)child);
	}

	public XmlMarshallable getAttributes(XmlAttributes attributes) throws SAXException {
		className = attributes.getValue("class");
		return this;
	}

	public void marshall(XmlOutputter output) {
		output.startTag(TAG);
		output.addAttribute("class",className);
		for(Iterator<ParamModel> it=params.iterator();it.hasNext();)
			it.next().marshall(output);
		output.endTag(TAG);
	}

	public void validate() throws InvalidXmlDefinition {
	}
	public ConfigurationModel clone() {
		ConfigurationModel cm = new ConfigurationModel(QN);
		cm.className=this.className;
		for(Iterator<ParamModel> it=params.iterator();it.hasNext();)
			cm.params.add(it.next());
		return cm;
	}

	public String getClassName() {
		return className;
	}

	public void setClassName(String className) {
		this.className = className;
	}

	public ArrayList<ParamModel> getParams() {
		return params;
	}

	public void setParams(ArrayList<ParamModel> params) {
		this.params = params;
	}

    public XmlMarshallable getChildToModify(String uri, String localName, String qName, Attributes atts) {
        return null;
    }

    public QName getQName() {
        return QN;
    }

}
